unit fmACLEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  SvCom_AclCenter, StdCtrls, ComCtrls, SvCom_SidCenter, SvCom_AclAPI;

type
    TACLEditorForm = class(TForm)
        LvACL: TListView;
        BnAdd: TButton;
        BnDelete: TButton;
        BnMerge: TButton;
        Label1: TLabel;
        ACLC: TsvAclCenter;
        SidC: TsvSidCenter;
        procedure LvACLChange(Sender: TObject; Item: TListItem;
          Change: TItemChange);
        procedure BnAddClick(Sender: TObject);
        procedure BnMergeClick(Sender: TObject);
        procedure BnDeleteClick(Sender: TObject);
    private
        { Private declarations }
        procedure AddListItem(EA: PExplicitAccess);
    public
        { Public declarations }
    end;

var
  ACLEditorForm: TACLEditorForm;

implementation

uses fmUserList;

{$R *.DFM}

procedure TACLEditorForm.LvACLChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
    BnDelete.Enabled:=LvACL.SelCount > 0;
end;

procedure TACLEditorForm.BnAddClick(Sender: TObject);
var ULF: TUsersListForm;
    i: Integer;
    LI: TListItem;
    EA: PExplicitAccess;
begin
    ULF:=TUsersListForm.Create(nil);
    try
        if ULF.ShowModal <> mrOK then exit;
        for i:=0 to ULF.LVSelected.Items.Count-1 do
        begin
            LI:=ULF.LVSelected.Items[i];
            EA:=ACLC.AddEntry;
            SidC.Sid:=LI.Data;
            EA.Trustee.ptstrName:=SidC.GetSidCopy;
            EA.grfAccessMode:=ULF.AccessMode;
            EA.grfAccessPermissions:=ULF.AccessMask;
            AddListItem(EA);
        end;
    finally
        ULF.Release;
    end;
end;

procedure TACLEditorForm.AddListItem(EA: PExplicitAccess);
var LI: TListItem;
    i: Integer;
    S: String;
    j: DWORD;
begin
    LI:=LvACL.Items.Add;

    if EA.Trustee.TrusteeForm = TRUSTEE_IS_SID then // two forms are possible
    begin
        SIDC.SID:=EA.Trustee.ptstrName; // SIDCenter will get info for us.
        LI.Caption:=SIDC.Domain+'\'+SIDC.UserName;
    end else
    begin
        LI.Caption:=StrPas(EA.Trustee.ptstrName);
    end;

    case EA.grfAccessMode of
        GRANT_ACCESS: LI.SubItems.Add('Grant');
        SET_ACCESS: LI.SubItems.Add('Set');
        DENY_ACCESS: LI.SubItems.Add('Deny');
        REVOKE_ACCESS: LI.SubItems.Add('Revoke');
    else
        LI.SubItems.Add('???'); // It is error
    end;

    S:='';
    j:=1;
    for i:=1 to 32 do
    begin
        if (EA.grfAccessPermissions and j) <> 0 then
            S:='1'+S
        else
            S:='0'+S;
        j:=j shl 1;
    end;
    LI.SubItems.Add(S);

    LI.Data:=EA;
end;

procedure TACLEditorForm.BnMergeClick(Sender: TObject);
var i: Integer;
    EA: PExplicitAccess;
begin
    ACLC.ACL:=ACLC.GetACLCopy;
    LvACL.Items.Clear;
    for i:=0 to ACLC.EntryCount-1 do
    begin
        EA:=ACLC.Entries[i];
        if EA.grfAccessMode = SET_ACCESS then EA.grfAccessMode:=GRANT_ACCESS;
        if EA.grfAccessMode = REVOKE_ACCESS then EA.grfAccessMode:=DENY_ACCESS;
        AddListItem(EA);
    end;
end;

procedure TACLEditorForm.BnDeleteClick(Sender: TObject);
var i: Integer;
    L: TList;
    LI: TListItem;
begin
    L:=TList.Create;
    try
        // Place all selected items into list
        for i:=0 to LvACL.Items.Count-1 do
        begin
            LI:=LvACL.Items[i];
            if not LI.Selected then continue;
            L.Add(LI);
        end;

        // Delete all selected items
        // with associated access entries
        for i:=L.Count-1 downto 0 do
        begin
            LI:=TListItem(L[i]);
            ACLC.DeleteEntry(LI.Index);
            LI.Free;
        end;
    finally
        L.Free;
    end;
end;

end.
